"use strict";
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
var compositionApi = require("@vue/composition-api");
var asyncQueueChain = require("async-queue-chain");
const stringTransformer = {
  fromQuery(value) {
    return value || void 0;
  },
  toQuery(value) {
    return value;
  }
};
const integerTransformer = {
  fromQuery(value) {
    const numberValue = Number.parseInt(value, 10);
    if (!Number.isInteger(numberValue)) {
      return void 0;
    }
    return numberValue;
  },
  toQuery(value) {
    return value == null ? void 0 : value.toString();
  }
};
const floatTransformer = {
  fromQuery(value) {
    const numberValue = Number.parseFloat(value);
    if (Number.isNaN(numberValue)) {
      return void 0;
    }
    return numberValue;
  },
  toQuery(value) {
    return value == null ? void 0 : value.toString();
  }
};
const booleanTransformer = {
  fromQuery(value) {
    if (value === "true") {
      return true;
    }
    if (value === "false") {
      return false;
    }
    return void 0;
  },
  toQuery(value) {
    return value == null ? void 0 : value.toString();
  }
};
function enumTransformer(enumObject) {
  return {
    fromQuery(key) {
      return enumObject[key];
    },
    toQuery(value) {
      const entry = Object.entries(enumObject).find(([, val]) => val === value);
      return entry ? entry[0] : void 0;
    }
  };
}
function useRouter() {
  const vm = compositionApi.getCurrentInstance();
  if (!vm) {
    throw new ReferenceError("Not found vue instance.");
  }
  return vm.proxy.$router;
}
function useRoute() {
  const vm = compositionApi.getCurrentInstance();
  if (!vm) {
    throw new ReferenceError("Not found vue instance.");
  }
  return compositionApi.computed(() => vm.proxy.$route);
}
function removeEmptyValues(object) {
  return Object.fromEntries(Object.entries(object).filter(([, value]) => !isEmpty(value)));
}
function isEmpty(value) {
  return value === null || value === void 0 || value === "";
}
function useRouteQuery(key, defaultValue, transformer) {
  const route = useRoute();
  const router = useRouter();
  function updateQueryParam(newValue) {
    queueQueryUpdate(router, route.value, key, newValue);
  }
  return compositionApi.computed({
    get() {
      if (!(key in route.value.query)) {
        return defaultValue;
      }
      const value = Array.isArray(route.value.query[key]) ? route.value.query[key][0] : route.value.query[key];
      if (!value) {
        return defaultValue;
      }
      if (!transformer) {
        return value;
      }
      const transformedValue = transformer.fromQuery(value);
      if (transformedValue === void 0) {
        return defaultValue;
      }
      return transformedValue;
    },
    set(value) {
      if (!transformer) {
        updateQueryParam(value);
        return;
      }
      const transformedValue = transformer.toQuery(value != null ? value : void 0);
      updateQueryParam(transformedValue);
    }
  });
}
let queryReplaceQueue;
function queueQueryUpdate(router, route, key, newValue) {
  if (!queryReplaceQueue) {
    queryReplaceQueue = asyncQueueChain.createAsyncQueue();
  }
  queryReplaceQueue.add((previousQuery) => {
    const newQuery = __spreadProps(__spreadValues({}, previousQuery), {
      [key]: newValue
    });
    return router.replace({
      query: removeEmptyValues(newQuery)
    }).then((newRoute) => newRoute.query).catch(() => previousQuery);
  });
  void queryReplaceQueue.run(route.query);
}
exports.booleanTransformer = booleanTransformer;
exports.enumTransformer = enumTransformer;
exports.floatTransformer = floatTransformer;
exports.integerTransformer = integerTransformer;
exports.stringTransformer = stringTransformer;
exports.useRouteQuery = useRouteQuery;
